-----------------------------------------------------------------------------------------------
-- MaelstromHelper
-- this addon helps you finding the Weather Station assigned to you in the Maelstrom Authority fight
-- created by esk213 (esk213@googlemail.com)
-- Some ideas and basics of code were taken from OhmnaKnockbackAssist and OhmnaHelper
-----------------------------------------------------------------------------------------------
 
require "Window"
 
local MaelstromHelper = {} 
local version = 4

local Localization = {
	["Weather Station"] = { 	
		[1] = "Weather Station",  	-- English Name
		[2] = "Wetterstation",  	-- German Name
	},
	["Maelstrom Authority"] = { 
		[1] = "Maelstrom Authority",-- English Name
		[2] = "Mahlstromgewalt",  	-- German Name
	},
}

function MaelstromHelper:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function MaelstromHelper:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
function MaelstromHelper:OnLoad()
    self.xmlDoc = XmlDoc.CreateFromFile("MaelstromHelper.xml")
	Apollo.LoadSprites("MHSprite.xml","MHSprite")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function MaelstromHelper:OnDocLoaded()
	aGeminiColor = Apollo.GetPackage("GeminiColor").tPackage
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.Overlay = Apollo.LoadForm(self.xmlDoc, "Overlay", "InWorldHudStratum", self)
	    self.PopUp = Apollo.LoadForm(self.xmlDoc, "MHPopUp", nil, self)
		if self.Overlay == nil then
			Print("Error")
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
	    self.PopUp:Show(false, true)
	    self.Overlay:Show(true)
				
		Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)
    	Apollo.RegisterSlashCommand("mhelper", "OnOptions", self)
    	Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
		Apollo.RegisterEventHandler("UnitDestroyed", "OnUnitDestroyed", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("MHInterfaceClick", "MHOnInterfaceClick", self)
		
		self.Checkboxes = {}
		self.Checkboxes[1] = self.PopUp:FindChild("NEButton")
		self.Checkboxes[2] = self.PopUp:FindChild("SWButton")
				
		self.WeatherStations = {}
		self.bossList = nil
		self.ownTarget = nil
		if self.lineColor == nil then
			self.lineColor = "500000ff"
		end
		
		
		self:FillOptionsWindow()

		local strCancel = Apollo.GetString(1)
		if strCancel == "Abbrechen" then
			self.Language = 2
		else
			self.Language = 1
		end

		self.player = GameLib.GetPlayerUnit()
	end
end

function MaelstromHelper:OnInterfaceMenuListHasLoaded()
		Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "MaelstromHelper", {"MHInterfaceClick", "", ""})
end

function MaelstromHelper:MHOnInterfaceClick()
	self:OnOptions()
end

function MaelstromHelper:OnOK()
	self.PopUp:Close()
end

function MaelstromHelper:OnCancel()
	self.PopUp:Close()
end


function MaelstromHelper:OnOptions()
	if not self.PopUp:IsVisible() then
		self.PopUp:Invoke()
	else
		self:OnCancel()
	end
end

function MaelstromHelper:OnSettingChange( wndHandler, wndControl, eMouseButton )
	self.Buttons[1] = self.Checkboxes[1]:IsChecked()
	self.Buttons[2] = self.Checkboxes[2]:IsChecked()
end

function MaelstromHelper:OnColorChange()	
	aGeminiColor:ShowColorPicker(self, self.OnGeminiColor, true, self.lineColor)
end

function MaelstromHelper:ClearMarks( wndHandler, wndControl, eMouseButton )
	self.Checkboxes[1]:SetCheck(false)
	self.Checkboxes[2]:SetCheck(false)
	self:OnSettingChange()
end

local MaelstromHelperInst = MaelstromHelper:new()
MaelstromHelperInst:Init()

function MaelstromHelper:OnGeminiColor(strColor)
	if strColor then
		self.lineColor = strColor
	end
end

function MaelstromHelper:OnNextFrame()
	self.player = GameLib.GetPlayerUnit()
	if self.player == nil then return end
	
	self.Overlay:DestroyAllPixies()
	
	if not self.player:IsInCombat() then
	  	Apollo.RemoveEventHandler("NextFrame",self)
		self.WeatherStations = nil 
		self.WeatherStations = {}
		self.bossList = nil 
		self.ownTarget = nil
		return
	end
		
	if self.ownTarget then
		self:SetupLines(self.ownTarget)
	end
end

function MaelstromHelper:OnEnteredCombat(unit, bInCombat)
	if unit and unit:GetType() ~= "NonPlayer" then return end
	local uName = unit:GetName()
	if uName == self:Localize("Weather Station") then
		Apollo.RegisterEventHandler("NextFrame", "OnNextFrame", self)
		self:CheckWeatherStations(unit)
	end
	if uName == self:Localize("Maelstrom Authority") then
		self.bossList = unit
	end
end

function MaelstromHelper:OnUnitCreated(unit)
	if unit and unit:GetType() ~= "NonPlayer" then return end
	local uName = unit:GetName()
		if uName == self:Localize("Weather Station") then
		self:CheckWeatherStations(unit)
	end
	if uName == self:Localize("Maelstrom Authority") then
		self.bossList = unit
	end
	local count = 0
	for _ in pairs(self.WeatherStations) do count = count + 1 end
	if count == 2 and self.bossList then
		self:FindTarget()
	end
end

function MaelstromHelper:OnUnitDestroyed(unit)
	if unit and unit:GetType() ~= "NonPlayer" then return end
	local uName = unit:GetName()
	if uName == self:Localize("Weather Station") then
		self:UpdateWeatherStations(unit)
		if unit == self.ownTarget then
			self.ownTarget = nil
		end
	end
end

function MaelstromHelper:Localize(name)
	if Localization[name] ~= nil then
		return Localization[name][self.Language]
	end
	return "Localization failed"
end

function MaelstromHelper:FindTarget()	
	local targets = {}
	targets = nil
	targets = {}
	
	local mid = self.bossList:GetPosition()
	
	for k,v in pairs(self.WeatherStations) do
		local pos = self.WeatherStations[k]:GetPosition()

		if pos.x < mid.x and pos.z < mid.z then --NW
			targets[1] = self.WeatherStations[k]
		elseif pos.x > mid.x and pos.z < mid.z then --NE
			targets[2] = self.WeatherStations[k]
		elseif pos.x < mid.x and pos.z > mid.z then --SW
			targets[3] = self.WeatherStations[k]
		elseif pos.x > mid.x and pos.z > mid.z then --SE
			targets[4] = self.WeatherStations[k]
		end
	end
	
	--NorthEast
	if self.Buttons[1] and not self.Buttons[2] then
		if targets[2] then
			self.ownTarget = targets[2]
		elseif targets[1] then
			self.ownTarget = targets[1]
		elseif targets[4] then
			self.ownTarget = targets[4]
		end
	--SouthWest
	elseif not self.Buttons[1] and self.Buttons[2] then
		if targets[3] then
			self.ownTarget = targets[3]
		elseif targets[4] then
			self.ownTarget = targets[4]
		elseif targets[1] then
			self.ownTarget = targets[1]
		end
	--Error
	else
		--Print("Multiple or no options selected!")
	end
end

function MaelstromHelper:CheckWeatherStations(unit)
	if not self.WeatherStations[unit:GetId()] then
		self.WeatherStations[unit:GetId()] = unit
	end
end

function MaelstromHelper:UpdateWeatherStations(unit)
	if self.WeatherStations[unit:GetId()] then
		self.WeatherStations[unit:GetId()] = nil
	end
end

function MaelstromHelper:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	local tsave = {}
	tsave.Buttons = {}
	tsave.Buttons[1] = self.Buttons[1]
	tsave.Buttons[2] = self.Buttons[2]
	tsave.lineColor = self.lineColor
	return tsave
end

function MaelstromHelper:OnRestore(eLevel, saveData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	for k,v in pairs(saveData) do
	   	self[k]=v
	end
end

function MaelstromHelper:FillOptionsWindow()
	if self.Buttons == nil then 
		self.Buttons = {}
		self.Buttons[1] = self.Checkboxes[1]:IsChecked()
		self.Buttons[2] = self.Checkboxes[2]:IsChecked()
	end
	
	self.Checkboxes[1]:SetCheck(self.Buttons[1])
	self.Checkboxes[2]:SetCheck(self.Buttons[2])
end

function MaelstromHelper:SetupLines(targ)
	if targ:IsDead() or not targ:IsValid() then
		return
	end
				
	local targetpos = targ:GetPosition()
	local startPoint = GameLib.WorldLocToScreenPoint(Vector3.New(targetpos.x,targetpos.y,targetpos.z))
	
	local ownpos = self.player:GetPosition()
	local endPoint = GameLib.WorldLocToScreenPoint(Vector3.New(ownpos.x,ownpos.y,ownpos.z))
	
	local linewidth = 20
	
	self.Overlay:AddPixie( { bLine = true, fWidth = linewidth, cr = self.lineColor, loc = { fPoints = { 0, 0, 0, 0 }, nOffsets = { startPoint.x, startPoint.y, endPoint.x, endPoint.y } } })
end
